@@ -155,6 +155,7 @@ pango_font_mask_get_type (void)
       { PANGO_FONT_MASK_WEIGHT, "PANGO_FONT_MASK_WEIGHT", "weight" },
       { PANGO_FONT_MASK_STRETCH, "PANGO_FONT_MASK_STRETCH", "stretch" },
       { PANGO_FONT_MASK_SIZE, "PANGO_FONT_MASK_SIZE", "size" },
+      { PANGO_FONT_MASK_GRAVITY, "PANGO_FONT_MASK_GRAVITY", "gravity" },
       { 0, NULL, NULL }
     };
     etype = g_flags_register_static (g_intern_static_string ("PangoFontMask"), values);
@@ -297,6 +298,12 @@ pango_script_get_type (void)
       { PANGO_SCRIPT_SYLOTI_NAGRI, "PANGO_SCRIPT_SYLOTI_NAGRI", "syloti-nagri" },
       { PANGO_SCRIPT_OLD_PERSIAN, "PANGO_SCRIPT_OLD_PERSIAN", "old-persian" },
       { PANGO_SCRIPT_KHAROSHTHI, "PANGO_SCRIPT_KHAROSHTHI", "kharoshthi" },
+      { PANGO_SCRIPT_UNKNOWN, "PANGO_SCRIPT_UNKNOWN", "unknown" },
+      { PANGO_SCRIPT_BALINESE, "PANGO_SCRIPT_BALINESE", "balinese" },
+      { PANGO_SCRIPT_CUNEIFORM, "PANGO_SCRIPT_CUNEIFORM", "cuneiform" },
+      { PANGO_SCRIPT_PHOENICIAN, "PANGO_SCRIPT_PHOENICIAN", "phoenician" },
+      { PANGO_SCRIPT_PHAGS_PA, "PANGO_SCRIPT_PHAGS_PA", "phags-pa" },
+      { PANGO_SCRIPT_NKO, "PANGO_SCRIPT_NKO", "nko" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static (g_intern_static_string ("PangoScript"), values);
@@ -339,6 +346,23 @@ pango_direction_get_type (void)
   }
   return etype;
 }
+GType
+pango_gravity_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { PANGO_GRAVITY_SOUTH, "PANGO_GRAVITY_SOUTH", "south" },
+      { PANGO_GRAVITY_EAST, "PANGO_GRAVITY_EAST", "east" },
+      { PANGO_GRAVITY_NORTH, "PANGO_GRAVITY_NORTH", "north" },
+      { PANGO_GRAVITY_WEST, "PANGO_GRAVITY_WEST", "west" },
+      { PANGO_GRAVITY_AUTO, "PANGO_GRAVITY_AUTO", "auto" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static (g_intern_static_string ("PangoGravity"), values);
+  }
+  return etype;
+}
 
 /* Generated data ends here */
 
