@@ -79,7 +79,8 @@ typedef enum {
   PANGO_FONT_MASK_VARIANT = 1 << 2,
   PANGO_FONT_MASK_WEIGHT  = 1 << 3,
   PANGO_FONT_MASK_STRETCH = 1 << 4,
-  PANGO_FONT_MASK_SIZE    = 1 << 5
+  PANGO_FONT_MASK_SIZE    = 1 << 5,
+  PANGO_FONT_MASK_GRAVITY = 1 << 6
 } PangoFontMask;
 
 /* CSS scale factors (1.2 factor between each size) */
@@ -131,6 +132,9 @@ gint                 pango_font_description_get_size          (const PangoFontDe
 void                 pango_font_description_set_absolute_size (PangoFontDescription *desc,
 							       double                size);
 gboolean             pango_font_description_get_size_is_absolute (const PangoFontDescription *desc);
+void                 pango_font_description_set_gravity       (PangoFontDescription *desc,
+							       PangoGravity          gravity);
+PangoGravity         pango_font_description_get_gravity       (const PangoFontDescription *desc);
 
 PangoFontMask pango_font_description_get_set_fields (const PangoFontDescription *desc);
 void          pango_font_description_unset_fields   (PangoFontDescription       *desc,
@@ -304,6 +308,7 @@ struct _PangoFontFaceClass
 GType                 pango_font_get_type          (void) G_GNUC_CONST;
 
 PangoFontDescription *pango_font_describe          (PangoFont        *font);
+PangoFontDescription *pango_font_describe_with_absolute_size (PangoFont        *font);
 PangoCoverage *       pango_font_get_coverage      (PangoFont        *font,
 						    PangoLanguage    *language);
 PangoEngineShape *    pango_font_find_shaper       (PangoFont        *font,
@@ -349,12 +354,12 @@ struct _PangoFontClass
   PangoFontMetrics *    (*get_metrics)        (PangoFont      *font,
 					       PangoLanguage  *language);
   PangoFontMap *        (*get_font_map)       (PangoFont      *font);
+  PangoFontDescription *(*describe_absolute)  (PangoFont      *font);
   /*< private >*/
 
   /* Padding for future expansion */
   void (*_pango_reserved1) (void);
   void (*_pango_reserved2) (void);
-  void (*_pango_reserved3) (void);
 };
 
 /* used for very rare and miserable situtations that we cannot even
@@ -365,6 +370,13 @@ struct _PangoFontClass
 
 #endif /* PANGO_ENABLE_BACKEND */
 
+#if defined(PANGO_ENABLE_ENGINE) || defined(PANGO_ENABLE_BACKEND)
+#define PANGO_GLYPH_EMPTY           ((PangoGlyph)0x0FFFFFFF)
+#define PANGO_GLYPH_UNKNOWN_FLAG    ((PangoGlyph)0x10000000)
+#define PANGO_GET_UNKNOWN_GLYPH(wc) ((PangoGlyph)(wc)|PANGO_GLYPH_UNKNOWN_FLAG)
+#endif
+
+
 G_END_DECLS
 
 #endif /* __PANGO_FONT_H__ */
