@@ -163,7 +163,6 @@ _pango_glyph_item_iter_next_cluster (PangoGlyphItemIter *iter)
 {
   int glyph_index = iter->end_glyph;
   PangoGlyphString *glyphs = iter->glyph_item->glyphs;
-  int cluster;
   PangoItem *item = iter->glyph_item->item;
 
   if (LTR (iter->glyph_item))
@@ -183,7 +182,6 @@ _pango_glyph_item_iter_next_cluster (PangoGlyphItemIter *iter)
   
   if (LTR (iter->glyph_item))
     {
-      cluster = glyphs->log_clusters[glyph_index];
       while (TRUE)
 	{
 	  glyph_index++;
@@ -195,7 +193,7 @@ _pango_glyph_item_iter_next_cluster (PangoGlyphItemIter *iter)
 	      break;
 	    }
 	  
-	  if (glyphs->log_clusters[glyph_index] != cluster)
+	  if (item->offset + glyphs->log_clusters[glyph_index] != iter->start_index)
 	    {
 	      iter->end_index = item->offset + glyphs->log_clusters[glyph_index];
 	      iter->end_char += g_utf8_strlen (iter->text + iter->start_index,
@@ -206,7 +204,6 @@ _pango_glyph_item_iter_next_cluster (PangoGlyphItemIter *iter)
     }
   else			/* RTL */
     {
-      cluster = glyphs->log_clusters[glyph_index];
       while (TRUE)
 	{
 	  glyph_index--;
@@ -218,7 +215,7 @@ _pango_glyph_item_iter_next_cluster (PangoGlyphItemIter *iter)
 	      break;
 	    }
 	  
-	  if (glyphs->log_clusters[glyph_index] != cluster)
+	  if (item->offset + glyphs->log_clusters[glyph_index] != iter->start_index)
 	    {
 	      iter->end_index = item->offset + glyphs->log_clusters[glyph_index];
 	      iter->end_char += g_utf8_strlen (iter->text + iter->start_index,
@@ -238,7 +235,7 @@ _pango_glyph_item_iter_next_cluster (PangoGlyphItemIter *iter)
  * 
  * Moves the iterator to the preceding cluster in the glyph item.
  * 
- * Return value: %TRUE if the iterator was moved, %FALSE if we were already on the
+ * Return value: %TRUE if the iterator was movedn, %FALSE if we were already on the
  *  first cluster.
  **/
 gboolean
@@ -246,7 +243,6 @@ _pango_glyph_item_iter_prev_cluster (PangoGlyphItemIter *iter)
 {
   int glyph_index = iter->start_glyph;
   PangoGlyphString *glyphs = iter->glyph_item->glyphs;
-  int cluster;
   PangoItem *item = iter->glyph_item->item;
     
   if (LTR (iter->glyph_item))
@@ -267,7 +263,6 @@ _pango_glyph_item_iter_prev_cluster (PangoGlyphItemIter *iter)
   
   if (LTR (iter->glyph_item))
     {
-      cluster = glyphs->log_clusters[glyph_index - 1];
       while (TRUE)
 	{
 	  glyph_index--;
@@ -279,7 +274,7 @@ _pango_glyph_item_iter_prev_cluster (PangoGlyphItemIter *iter)
 	      break;
 	    }
 	  
-	  if (glyphs->log_clusters[glyph_index] != cluster)
+	  if (item->offset + glyphs->log_clusters[glyph_index] != iter->end_index)
 	    {
 	      iter->start_index = item->offset + glyphs->log_clusters[glyph_index];
 	      iter->start_char -= g_utf8_strlen (iter->text + iter->start_index,
@@ -290,7 +285,6 @@ _pango_glyph_item_iter_prev_cluster (PangoGlyphItemIter *iter)
     }
   else			/* RTL */
     {
-      cluster = glyphs->log_clusters[glyph_index + 1];
       while (TRUE)
 	{
 	  glyph_index++;
@@ -302,7 +296,7 @@ _pango_glyph_item_iter_prev_cluster (PangoGlyphItemIter *iter)
 	      break;
 	    }
 	  
-	  if (glyphs->log_clusters[glyph_index] != cluster)
+	  if (item->offset + glyphs->log_clusters[glyph_index] != iter->start_index)
 	    {
 	      iter->start_index = item->offset + glyphs->log_clusters[glyph_index];
 	      iter->start_char -= g_utf8_strlen (iter->text + iter->start_index,
@@ -319,14 +313,14 @@ _pango_glyph_item_iter_prev_cluster (PangoGlyphItemIter *iter)
 /**
  * _pango_glyph_item_iter_init_start:
  * @iter: pointer to a #PangoGlyphItemIter structure
- * @glyph_item: the glyph item that @iter points into
+ * @glyph_item: the glyph item that the iter points into
  * @text: text corresponding to the glyph item
  * 
  * Initializes a #PangoGlyphItemIter structure to point to the
  * first cluster in a glyph item.
  * 
  * Return value: %FALSE if there are no clusters in the glyph item;
- *  in this case, the state of @iter is undefined.
+ *  in this case, the state of the iter is undefined.
  **/
 gboolean
 _pango_glyph_item_iter_init_start (PangoGlyphItemIter  *iter,
@@ -349,16 +343,16 @@ _pango_glyph_item_iter_init_start (PangoGlyphItemIter  *iter,
 }
 
 /**
- * _pango_glyph_item_iter_init_start:
+ * _pango_glyph_item_iter_init_end:
  * @iter: pointer to a #PangoGlyphItemIter structure
- * @glyph_item: the glyph item that @iter points into
+ * @glyph_item: the glyph item that the iter points into
  * @text: text corresponding to the glyph item
  * 
  * Initializes a #PangoGlyphItemIter structure to point to the
  * last cluster in a glyph item.
  * 
  * Return value: %FALSE if there are no clusters in the glyph item;
- *  in this case, the state of @iter is undefined.
+ *  in this case, the state of the iter is undefined.
  **/
 gboolean
 _pango_glyph_item_iter_init_end (PangoGlyphItemIter  *iter,
@@ -397,7 +391,7 @@ append_attrs (PangoGlyphItem *glyph_item,
     g_slist_concat (glyph_item->item->analysis.extra_attrs, attrs);
 }
 
-/* Make a deep copy of a #GSList of PangoAttribute
+/* Make a deep copy of a GSlist of PangoAttribute
  */
 static GSList *
 attr_slist_copy (GSList *attrs)
@@ -453,7 +447,7 @@ split_before_cluster_start (ApplyAttrsState *state)
  *
  * All attributes that start or end inside a cluster are applied
  * to that cluster; for instance, if half of a cluster is underlined
- * and the other-half strikethrough, then the cluster will end
+ * and the other-half strikethough, then the cluster will end
  * up with both underline and strikethrough attributes. In these
  * cases, it may happen that item->extra_attrs for some of the
  * result items can have multiple attributes of the same type.
