@@ -26,6 +26,7 @@
 
 #define N_RENDER_PARTS 4
 
+#define PANGO_IS_RENDERER_FAST(renderer) (renderer != NULL)
 #define IS_VALID_PART(part) ((guint)part < N_RENDER_PARTS)
 
 typedef struct _LineState LineState;
@@ -175,7 +176,7 @@ pango_renderer_draw_layout (PangoRenderer    *renderer,
       PangoLayoutLine *line;
       int              baseline;
       
-      line = pango_layout_iter_get_line (iter);
+      line = pango_layout_iter_get_line_readonly (iter);
       
       pango_layout_iter_get_line_extents (iter, NULL, &logical_rect);
       baseline = pango_layout_iter_get_baseline (iter);
@@ -452,7 +453,7 @@ pango_renderer_draw_layout_line (PangoRenderer    *renderer,
   gboolean got_overall = FALSE;
   PangoRectangle overall_rect;
   
-  g_return_if_fail (PANGO_IS_RENDERER (renderer));
+  g_return_if_fail (PANGO_IS_RENDERER_FAST (renderer));
 
   /* We only change the matrix if the renderer isn't already
    * active.
@@ -477,7 +478,11 @@ pango_renderer_draw_layout_line (PangoRenderer    *renderer,
       gint rise;
       PangoLayoutRun *run = l->data;
       PangoAttrShape *shape_attr;
-      PangoRectangle ink_rect, logical_rect;
+      PangoRectangle ink_rect, *ink = NULL;
+      PangoRectangle logical_rect, *logical = NULL;
+
+      if (run->item->analysis.centered_baseline)
+        logical = &logical_rect;
 
       pango_renderer_prepare_run (renderer, run);
       
@@ -487,6 +492,8 @@ pango_renderer_draw_layout_line (PangoRenderer    *renderer,
 	{
 	  ink_rect = shape_attr->ink_rect;
 	  logical_rect = shape_attr->logical_rect;
+	  ink = &ink_rect;
+	  logical = &logical_rect;
 	  glyph_string_width = shape_attr->logical_rect.width;
 	}
       else
@@ -494,16 +501,24 @@ pango_renderer_draw_layout_line (PangoRenderer    *renderer,
 	  if (renderer->underline != PANGO_UNDERLINE_NONE ||
 	      renderer->strikethrough)
 	    {
-	      pango_glyph_string_extents (run->glyphs, run->item->analysis.font,
-					    &ink_rect, &logical_rect);
-	      glyph_string_width = logical_rect.width;
+	      ink = &ink_rect;
+	      logical = &logical_rect;
 	    }
+	  if (G_UNLIKELY (ink || logical))
+	    pango_glyph_string_extents (run->glyphs, run->item->analysis.font,
+					ink, logical);
+	  if (logical)
+	    glyph_string_width = logical_rect.width;
 	  else
 	    glyph_string_width = pango_glyph_string_get_width (run->glyphs);
 	}
 
       state.logical_rect_end = x + x_off + glyph_string_width;
 
+      if (run->item->analysis.centered_baseline)
+        rise += logical_rect.y + logical_rect.height / 2;
+
+
       if (renderer->priv->color_set[PANGO_RENDER_PART_BACKGROUND])
 	{
 	  if (!got_overall)
@@ -515,7 +530,7 @@ pango_renderer_draw_layout_line (PangoRenderer    *renderer,
 	  pango_renderer_draw_rectangle (renderer,
 					 PANGO_RENDER_PART_BACKGROUND,
 					 x + x_off,
-					 y + overall_rect.y,
+					 y - rise + overall_rect.y,
 					 glyph_string_width,
 					 overall_rect.height);
 	}
@@ -540,12 +555,12 @@ pango_renderer_draw_layout_line (PangoRenderer    *renderer,
 	  if (renderer->underline != PANGO_UNDERLINE_NONE)
 	    add_underline (renderer, &state,metrics,
 			   x + x_off, y - rise,
-			   &ink_rect, &logical_rect);
+			   ink, logical);
 	  
 	  if (renderer->strikethrough)
 	    add_strikethrough (renderer, &state, metrics,
 			       x + x_off, y - rise,
-			       &ink_rect, &logical_rect);
+			       ink, logical);
 	  
 	  pango_font_metrics_unref (metrics);
 	}
@@ -591,7 +606,7 @@ pango_renderer_draw_glyphs (PangoRenderer    *renderer,
 			    int               x,
 			    int               y)
 {
-  g_return_if_fail (PANGO_IS_RENDERER (renderer));
+  g_return_if_fail (PANGO_IS_RENDERER_FAST (renderer));
   
   pango_renderer_activate (renderer);
 
@@ -651,7 +666,7 @@ pango_renderer_draw_rectangle (PangoRenderer   *renderer,
 			       int              width,
 			       int              height)
 {
-  g_return_if_fail (PANGO_IS_RENDERER (renderer));
+  g_return_if_fail (PANGO_IS_RENDERER_FAST (renderer));
   g_return_if_fail (IS_VALID_PART (part));
   g_return_if_fail (renderer->active_count > 0);
   
@@ -791,7 +806,7 @@ pango_renderer_draw_error_underline (PangoRenderer *renderer,
 				     int            width,
 				     int            height)
 {
-  g_return_if_fail (PANGO_IS_RENDERER (renderer));
+  g_return_if_fail (PANGO_IS_RENDERER_FAST (renderer));
   g_return_if_fail (renderer->active_count > 0);
   
   PANGO_RENDERER_GET_CLASS (renderer)->draw_error_underline (renderer, x, y, width, height);
@@ -939,7 +954,7 @@ pango_renderer_draw_trapezoid (PangoRenderer  *renderer,
 			       double          x12,
 			       double          x22)
 {
-  g_return_if_fail (PANGO_IS_RENDERER (renderer));
+  g_return_if_fail (PANGO_IS_RENDERER_FAST (renderer));
   g_return_if_fail (renderer->active_count > 0);
   
   if (PANGO_RENDERER_GET_CLASS (renderer)->draw_trapezoid)
@@ -967,7 +982,7 @@ pango_renderer_draw_glyph (PangoRenderer *renderer,
 			   double         x,
 			   double         y)
 {
-  g_return_if_fail (PANGO_IS_RENDERER (renderer));
+  g_return_if_fail (PANGO_IS_RENDERER_FAST (renderer));
   g_return_if_fail (renderer->active_count > 0);
 	  
   if (glyph == PANGO_GLYPH_EMPTY) /* glyph PANGO_GLYPH_EMPTY never renders */
@@ -994,7 +1009,7 @@ pango_renderer_draw_glyph (PangoRenderer *renderer,
 void
 pango_renderer_activate (PangoRenderer *renderer)
 {
-  g_return_if_fail (PANGO_IS_RENDERER (renderer));
+  g_return_if_fail (PANGO_IS_RENDERER_FAST (renderer));
 
   renderer->active_count++;
   if (renderer->active_count == 1)
@@ -1016,7 +1031,7 @@ pango_renderer_activate (PangoRenderer *renderer)
 void
 pango_renderer_deactivate (PangoRenderer *renderer)
 {
-  g_return_if_fail (PANGO_IS_RENDERER (renderer));
+  g_return_if_fail (PANGO_IS_RENDERER_FAST (renderer));
   g_return_if_fail (renderer->active_count > 0);
   
   if (renderer->active_count == 1)
@@ -1042,7 +1057,7 @@ pango_renderer_set_color (PangoRenderer    *renderer,
 			  PangoRenderPart   part,
 			  const PangoColor *color)
 {
-  g_return_if_fail (PANGO_IS_RENDERER (renderer));
+  g_return_if_fail (PANGO_IS_RENDERER_FAST (renderer));
   g_return_if_fail (IS_VALID_PART (part));
 
   if ((!color && !renderer->priv->color_set[part]) ||
@@ -1082,7 +1097,7 @@ PangoColor *
 pango_renderer_get_color (PangoRenderer   *renderer,
 			  PangoRenderPart  part)
 {
-  g_return_val_if_fail (PANGO_IS_RENDERER (renderer), NULL);
+  g_return_val_if_fail (PANGO_IS_RENDERER_FAST (renderer), NULL);
   g_return_val_if_fail (IS_VALID_PART (part), NULL);
   
   if (renderer->priv->color_set[part])
@@ -1096,7 +1111,7 @@ pango_renderer_get_color (PangoRenderer   *renderer,
  * @renderer: a #PangoRenderer
  * @part: the part for which rendering has changed.
  * 
- * Informs Pango that the way that the rendering is done
+ * Informs Pango that the way that the renderering is done
  * for @part has changed in a way that would prevent multiple
  * pieces being joined together into one drawing call. For
  * instance, if a subclass of #PangoRenderer was to add a stipple
@@ -1116,7 +1131,7 @@ void
 pango_renderer_part_changed (PangoRenderer    *renderer,
 			     PangoRenderPart   part)
 {
-  g_return_if_fail (PANGO_IS_RENDERER (renderer));
+  g_return_if_fail (PANGO_IS_RENDERER_FAST (renderer));
   g_return_if_fail (IS_VALID_PART (part));
   g_return_if_fail (renderer->active_count > 0);
   
@@ -1131,7 +1146,7 @@ pango_renderer_part_changed (PangoRenderer    *renderer,
  * @renderer: a #PangoRenderer
  * @run: a #PangoLayoutRun
  * 
- * Set up the state of the #PangoRenderer for rendering @run.
+ * Set up the state of the PangoRenderer for rendering @run.
  *
  * Since: 1.8
  **/
@@ -1139,7 +1154,7 @@ static void
 pango_renderer_prepare_run (PangoRenderer  *renderer,
 			    PangoLayoutRun *run)
 {
-  g_return_if_fail (PANGO_IS_RENDERER (renderer));
+  g_return_if_fail (PANGO_IS_RENDERER_FAST (renderer));
   
   PANGO_RENDERER_GET_CLASS (renderer)->prepare_run (renderer, run);
 }
@@ -1218,7 +1233,7 @@ void
 pango_renderer_set_matrix (PangoRenderer     *renderer,
 			   const PangoMatrix *matrix)
 {
-  g_return_if_fail (PANGO_IS_RENDERER (renderer));
+  g_return_if_fail (PANGO_IS_RENDERER_FAST (renderer));
 
   if (renderer->matrix)
     pango_matrix_free (renderer->matrix);
