@@ -30,6 +30,7 @@
 #include "pangofc-private.h"
 #include "pango-impl-utils.h"
 #include "modules.h"
+#include "pango-enum-types.h"
 
 typedef struct _PangoFcCoverageKey  PangoFcCoverageKey;
 typedef struct _PangoFcFace         PangoFcFace;
@@ -213,6 +214,17 @@ pango_fc_font_map_class_init (PangoFcFontMapClass *class)
   g_type_class_add_private (object_class, sizeof (PangoFcFontMapPrivate));
 }
 
+static gpointer
+get_gravity_class (void)
+{
+  static GEnumClass *class = NULL;
+
+  if (G_UNLIKELY (!class))
+    class = g_type_class_ref (PANGO_TYPE_GRAVITY);
+
+  return class;
+}
+
 static guint
 pango_fc_pattern_hash (FcPattern *pattern)
 {
@@ -796,6 +808,8 @@ pango_fc_make_pattern (const  PangoFontDescription *description,
   FcPattern *pattern;
   int slant;
   int weight;
+  PangoGravity gravity;
+  FcBool vertical;
   char **families;
   int i;
 #ifdef FC_WIDTH
@@ -808,6 +822,20 @@ pango_fc_make_pattern (const  PangoFontDescription *description,
   width = pango_fc_convert_width_to_fc (pango_font_description_get_stretch (description));
 #endif
 
+  gravity = pango_font_description_get_gravity (description);
+  switch (gravity)
+    {
+      case PANGO_GRAVITY_SOUTH:
+      case PANGO_GRAVITY_NORTH:
+      default:
+        vertical = FcFalse;
+	break;
+      case PANGO_GRAVITY_EAST:
+      case PANGO_GRAVITY_WEST:
+        vertical = FcTrue;
+        break;
+    }
+
   /* The reason for passing in FC_SIZE as well as FC_PIXEL_SIZE is
    * to work around a bug in libgnomeprint where it doesn't look
    * for FC_PIXEL_SIZE. See http://bugzilla.gnome.org/show_bug.cgi?id=169020
@@ -821,6 +849,9 @@ pango_fc_make_pattern (const  PangoFontDescription *description,
 			    FC_SLANT,  FcTypeInteger, slant,
 #ifdef FC_WIDTH
 			    FC_WIDTH,  FcTypeInteger, width,
+#endif
+#ifdef FC_VERTICAL_LAYOUT
+			    FC_VERTICAL_LAYOUT,  FcTypeBool, vertical,
 #endif
  			    FC_SIZE,  FcTypeDouble,  pixel_size * (72. / dpi),
  			    FC_PIXEL_SIZE,  FcTypeDouble,  pixel_size,
@@ -835,6 +866,12 @@ pango_fc_make_pattern (const  PangoFontDescription *description,
 
   if (language)
     FcPatternAddString (pattern, FC_LANG, (FcChar8 *) pango_language_to_string (language));
+
+  if (gravity != PANGO_GRAVITY_SOUTH)
+    {
+      GEnumValue *value = g_enum_get_value (get_gravity_class (), gravity);
+      FcPatternAddString (pattern, PANGO_FC_GRAVITY, value->value_nick);
+    }
   
   return pattern;
 }
@@ -881,7 +918,7 @@ pango_fc_font_map_new_font (PangoFontMap               *fontmap,
 	{
 	  FcMatrix fc_matrix;
 	  
-	  /* Fontconfig has the Y axis pointing up, Pango, down.
+	  /* FontConfig has the Y axis pointing up, Pango, down.
 	   */
 	  fc_matrix.xx = pango_matrix->xx;
 	  fc_matrix.xy = - pango_matrix->xy;
@@ -1436,7 +1473,7 @@ cleanup_font (gpointer        key,
  * 
  * Clears all cached information for the fontmap and marks
  * all fonts open for the fontmap as dead. (See the shutdown()
- * virtual function of #PangoFcFont.) This function might be used
+ * virtual function of PangoFcFont.) This function might be used
  * by a backend when the underlying windowing system for the font
  * map exits. This function is only intended to be called from
  * only for backend implementations deriving from #PangoFcFontmap.
@@ -1567,6 +1604,7 @@ pango_fc_font_description_from_pattern (FcPattern *pattern, gboolean include_siz
   PangoWeight weight;
   PangoStretch stretch;
   double size;
+  PangoGravity gravity;
   
   FcChar8 *s;
   int i;
@@ -1607,6 +1645,16 @@ pango_fc_font_description_from_pattern (FcPattern *pattern, gboolean include_siz
   if (include_size && FcPatternGetDouble (pattern, FC_SIZE, 0, &size) == FcResultMatch)
     pango_font_description_set_size (desc, size * PANGO_SCALE);
 
+  if (FcPatternGetString (pattern, PANGO_FC_GRAVITY, 0, (FcChar8 **)&s) == FcResultMatch)
+    {
+      GEnumValue *value = g_enum_get_value_by_nick (get_gravity_class (), s);
+      gravity = value->value;
+    }
+  else
+    gravity = PANGO_GRAVITY_SOUTH;
+
+  pango_font_description_set_gravity (desc, gravity);
+
   return desc;
 }
 
@@ -1662,6 +1710,10 @@ pango_fc_face_describe (PangoFontFace *face)
     {
       desc = pango_fc_font_description_from_pattern (result_pattern, FALSE);
       FcPatternDestroy (result_pattern);
+      /* Unset gravity.  We want gravity to be set to descriptions of fonts,
+       * but not faces.
+       */
+      pango_font_description_unset_fields (desc, PANGO_FONT_MASK_GRAVITY);
     }
 
   FcPatternDestroy (match_pattern);
@@ -1792,7 +1844,7 @@ pango_fc_face_get_type (void)
 
   if (!object_type)
     {
-      static const GTypeInfo object_info =
+      const GTypeInfo object_info =
       {
         sizeof (PangoFontFaceClass),
         (GBaseInitFunc) NULL,
@@ -1983,7 +2035,7 @@ pango_fc_family_get_type (void)
 
   if (!object_type)
     {
-      static const GTypeInfo object_info =
+      const GTypeInfo object_info =
       {
         sizeof (PangoFontFamilyClass),
         (GBaseInitFunc) NULL,
