@@ -180,7 +180,7 @@ pango_x_font_map_get_type (void)
 
   if (!object_type)
     {
-      static const GTypeInfo object_info =
+      const GTypeInfo object_info =
       {
         sizeof (PangoFontMapClass),
         (GBaseInitFunc) NULL,
@@ -524,11 +524,11 @@ pango_x_font_map_load_font (PangoFontMap               *fontmap,
 
 /* We need to be robust against errors accessing the coverage
  * cache window, since it is not our window. So we temporarily
- * set this error handler while accessing it. The error_occurred
- * global allows us to tell whether an error occurred for
+ * set this error handler while accessing it. The error_occured
+ * global allows us to tell whether an error occured for
  * XChangeProperty
  */
-static gboolean error_occurred;
+static gboolean error_occured;
 
 static int 
 ignore_error (Display     *d,
@@ -601,7 +601,7 @@ pango_x_real_get_coverage_win (Display *display)
   return retval;
 }
 
-/* Find or create the persistent window for caching font coverage
+/* Find or create the peristant window for caching font coverage
  * information.
  *
  * To assure atomic creation, we first look for the window, then if we
@@ -756,7 +756,7 @@ pango_x_store_cached_coverage (PangoXFontMap *xfontmap,
 	break;
 
       old_handler = XSetErrorHandler (ignore_error);
-      error_occurred = False;
+      error_occured = False;
   
       XChangeProperty (xfontmap->display,
 		       coverage_win,
@@ -768,7 +768,7 @@ pango_x_store_cached_coverage (PangoXFontMap *xfontmap,
       XSync (xfontmap->display, False);
       XSetErrorHandler (old_handler);
 
-      if (!error_occurred)
+      if (!error_occured)
 	break;
       else
 	{
@@ -896,8 +896,7 @@ pango_x_font_map_read_alias_file (PangoXFontMap *xfontmap,
     error:
       if (xface)
 	{
-	  if (xface->xlfd)
-	    g_free (xface->xlfd);
+	  g_free (xface->xlfd);
 	  if (xface->description)
 	    pango_font_description_free (xface->description);
 	  g_free (xface);
@@ -938,7 +937,7 @@ pango_x_font_map_read_aliases (PangoXFontMap *xfontmap)
 }
 
 /*
- * Returns TRUE if the fontname is a valid XLFD.
+ * Returns %TRUE if the fontname is a valid XLFD.
  * (It just checks if the number of dashes is 14, and that each
  * field < XLFD_MAX_FIELD_LEN  characters long - that's not in the XLFD but it
  * makes it easier for me).
@@ -1020,8 +1019,8 @@ pango_x_get_identifier (const char *fontname)
 
 /*
  * This fills the buffer with the specified field from the X Logical Font
- * Description name, and returns it. If fontname is NULL or the field is
- * longer than XFLD_MAX_FIELD_LEN it returns NULL.
+ * Description name, and returns it. If fontname is %NULL or the field is
+ * longer than XFLD_MAX_FIELD_LEN it returns %NULL.
  * Note: For the charset field, we also return the encoding, e.g. 'iso8859-1'.
  */
 static char*
@@ -1221,7 +1220,7 @@ match_end (const char *a, const char *b)
     return (strcmp (a + len_a - len_b, b) == 0);
 }
 
-/* Given a XLFD, charset and size, find the best matching installed X font.
+/* Given a xlfd, charset and size, find the best matching installed X font.
  * The XLFD must be a full XLFD (14 fields)
  */
 char *
@@ -1493,7 +1492,7 @@ pango_x_face_get_type (void)
 
   if (!object_type)
     {
-      static const GTypeInfo object_info =
+      const GTypeInfo object_info =
       {
         sizeof (PangoFontFaceClass),
         (GBaseInitFunc) NULL,
@@ -1652,7 +1651,7 @@ pango_x_family_get_type (void)
 
   if (!object_type)
     {
-      static const GTypeInfo object_info =
+      const GTypeInfo object_info =
       {
         sizeof (PangoFontFamilyClass),
         (GBaseInitFunc) NULL,
